"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _commandland = require("commandland");

var _dotStore = _interopRequireDefault(require("dot-store"));

var _dotStoreFs = _interopRequireDefault(require("dot-store-fs"));

var _inquirer = require("inquirer");

var _mri = _interopRequireDefault(require("mri"));

var _opn = _interopRequireDefault(require("opn"));

function taskEnv() {
  return _taskEnv.apply(this, arguments);
}

function _taskEnv() {
  _taskEnv = (0, _asyncToGenerator2.default)(function* (config = {}) {
    config = (0, _objectSpread2.default)({
      alias: {},
      customArgs: {},
      preSetup: [],
      setup: [],
      stores: {},
      tasks: [],
      teardown: [],
      urls: {}
    }, config);
    let {
      preSetup,
      setup
    } = config;
    config = yield runSetup({
      config,
      preSetup,
      setup
    });
    yield runTasks(config);
    let {
      teardown
    } = config;
    yield runFns({
      config,
      fns: teardown
    });
    return config;
  });
  return _taskEnv.apply(this, arguments);
}

function extractSetupFns(_x) {
  return _extractSetupFns.apply(this, arguments);
}

function _extractSetupFns() {
  _extractSetupFns = (0, _asyncToGenerator2.default)(function* (config) {
    let {
      tasks
    } = config;

    for (let pkg of tasks) {
      if (pkg.preSetup) {
        config.preSetup.push(pkg.preSetup);
      }

      if (pkg.setup) {
        config.setup.push(pkg.setup);
      }

      if (pkg.teardown) {
        config.setup.push(pkg.teardown);
      }
    }

    return config;
  });
  return _extractSetupFns.apply(this, arguments);
}

function help(_x2) {
  return _help.apply(this, arguments);
}

function _help() {
  _help = (0, _asyncToGenerator2.default)(function* ({
    arg,
    config,
    pkg
  }) {
    let {
      parsedArgs,
      urls
    } = config;
    let {
      h
    } = parsedArgs;

    if (h) {
      let url = urls[arg];

      if (url) {
        yield (0, _opn.default)(url, {
          wait: false
        });
      } else if (pkg.help) {
        yield pkg.help(parsedArgs, config);
      }
    }

    return h;
  });
  return _help.apply(this, arguments);
}

function namespaceAliases(config) {
  config.alias = {
    default: (0, _objectSpread2.default)({
      h: ["help"]
    }, config.alias)
  };
  return config;
}

function parseArgs(_x3) {
  return _parseArgs.apply(this, arguments);
}

function _parseArgs() {
  _parseArgs = (0, _asyncToGenerator2.default)(function* (config) {
    let {
      args,
      tasks
    } = config;
    let options = (0, _mri.default)(args);
    let alias = yield setupAlias({
      config,
      options
    });
    options = (0, _mri.default)(args, {
      alias
    });

    if (!options._.length) {
      options._ = ["defaultTask"];
    }

    let ask = (0, _inquirer.createPromptModule)();
    let stores = yield setupStores(config);
    let parsedArgs = (0, _objectSpread2.default)({}, config.customArgs, stores, {
      ask,
      run: _commandland.run
    });
    parsedArgs.tasks = yield setupTasks({
      parsedArgs,
      tasks
    });
    return (0, _objectSpread2.default)({}, options, parsedArgs);
  });
  return _parseArgs.apply(this, arguments);
}

function runFns(_x4) {
  return _runFns.apply(this, arguments);
}

function _runFns() {
  _runFns = (0, _asyncToGenerator2.default)(function* ({
    config,
    fns
  }) {
    if (fns) {
      for (let fn of fns) {
        config = (yield fn(config, config.parsedArgs)) || config;
      }
    }

    return config;
  });
  return _runFns.apply(this, arguments);
}

function runSetup(_x5) {
  return _runSetup.apply(this, arguments);
}

function _runSetup() {
  _runSetup = (0, _asyncToGenerator2.default)(function* ({
    config,
    preSetup,
    setup
  }) {
    config = namespaceAliases(config);
    config = yield extractSetupFns(config);
    config = yield setupArgs(config);
    config = yield runFns({
      config,
      fns: preSetup
    });
    config = yield setupArgs(config);
    config = yield runFns({
      config,
      fns: setup
    });
    config = yield setupArgs(config);
    return config;
  });
  return _runSetup.apply(this, arguments);
}

function runTasks(_x6) {
  return _runTasks.apply(this, arguments);
}

function _runTasks() {
  _runTasks = (0, _asyncToGenerator2.default)(function* (config) {
    let {
      parsedArgs,
      tasks
    } = config;

    for (let arg of parsedArgs._) {
      let found = false;

      for (let pkg of tasks) {
        if (pkg[arg]) {
          found = true;

          if (!(yield help({
            arg,
            config,
            pkg
          }))) {
            yield pkg[arg](parsedArgs, config);
          }
        }
      }

      if (!found) {
        throw new Error(`task "${arg}" not found`);
      }
    }
  });
  return _runTasks.apply(this, arguments);
}

function setupAlias(_x7) {
  return _setupAlias.apply(this, arguments);
}

function _setupAlias() {
  _setupAlias = (0, _asyncToGenerator2.default)(function* ({
    config,
    options
  }) {
    let alias = config.alias["default"];

    for (let arg of options._) {
      let argAlias = config.alias[arg];

      if (argAlias) {
        for (let key in argAlias) {
          if (alias[key]) {
            argAlias[key] = alias[key].concat(argAlias[key]);
          }
        }

        alias = (0, _objectSpread2.default)({}, alias, argAlias);
      }
    }

    return alias;
  });
  return _setupAlias.apply(this, arguments);
}

function setupArgs(_x8) {
  return _setupArgs.apply(this, arguments);
}

function _setupArgs() {
  _setupArgs = (0, _asyncToGenerator2.default)(function* (config) {
    return (0, _objectSpread2.default)({}, config, {
      parsedArgs: yield parseArgs(config)
    });
  });
  return _setupArgs.apply(this, arguments);
}

function setupStores(_x9) {
  return _setupStores.apply(this, arguments);
}

function _setupStores() {
  _setupStores = (0, _asyncToGenerator2.default)(function* (config) {
    let {
      parsedArgs = {},
      stores
    } = config;
    let args = {};

    for (let store in stores) {
      args[store] = parsedArgs[store] || (yield (0, _dotStoreFs.default)(new _dotStore.default(), stores[store]));
    }

    return args;
  });
  return _setupStores.apply(this, arguments);
}

function setupTasks(_x10) {
  return _setupTasks.apply(this, arguments);
}

function _setupTasks() {
  _setupTasks = (0, _asyncToGenerator2.default)(function* ({
    parsedArgs,
    tasks
  }) {
    let boundTasks = {};

    for (let pkg of tasks) {
      for (let task in pkg) {
        let taskFn = pkg[task];

        boundTasks[task] = args => {
          return taskFn((0, _objectSpread2.default)({}, parsedArgs, args));
        };
      }
    }

    return boundTasks;
  });
  return _setupTasks.apply(this, arguments);
}

module.exports = taskEnv;
//# sourceMappingURL=index.js.map